@@ -1,19 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * Copyright (c) 2000-2002,2005 Silicon Graphics, Inc.
  * All Rights Reserved.
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it would be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write the Free Software Foundation,
- * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "libxfs.h"
@@ -22,6 +10,7 @@
 #include "faddr.h"
 #include "fprint.h"
 #include "field.h"
+#include "dquot.h"
 #include "inode.h"
 #include "io.h"
 #include "output.h"
@@ -220,6 +209,19 @@ push_cur(void)
 	cur_typ = NULL;
 }
 
+void
+push_cur_and_set_type(void)
+{
+	/* save current state */
+	push_cur();
+	if (iocur_top[-1].typ && iocur_top[-1].typ->typnm == TYP_INODE)
+		set_cur_inode(iocur_top[-1].ino);
+	else
+		set_cur(iocur_top[-1].typ, iocur_top[-1].bb,
+			iocur_top[-1].blen, DB_RING_IGN,
+			iocur_top[-1].bbmap);
+}
+
 static int
 push_f(
 	int		argc,
@@ -240,14 +242,7 @@ push_f(
 		}
 	}
 
-	/* save current state */
-	push_cur();
-	if (iocur_top[-1].typ && iocur_top[-1].typ->typnm == TYP_INODE)
-		set_cur_inode(iocur_top[-1].ino);
-	else
-		set_cur(iocur_top[-1].typ, iocur_top[-1].bb,
-			iocur_top[-1].blen, DB_RING_IGN,
-			iocur_top[-1].bbmap);
+	push_cur_and_set_type();
 
 	/* run requested command */
 	if (argc>1)
@@ -465,28 +460,6 @@ xfs_dummy_verify(
 	return;
 }
 
-void
-xfs_verify_recalc_inode_crc(
-	struct xfs_buf *bp)
-{
-	ASSERT(iocur_top->ino_buf);
-	ASSERT(iocur_top->bp == bp);
-
-	libxfs_dinode_calc_crc(mp, iocur_top->data);
-	iocur_top->ino_crc_ok = 1;
-}
-
-void
-xfs_verify_recalc_dquot_crc(
-	struct xfs_buf *bp)
-{
-	ASSERT((iocur_top->dquot_buf));
-	ASSERT(iocur_top->bp == bp);
-
-	xfs_update_cksum(iocur_top->data, sizeof(struct xfs_dqblk),
-			 XFS_DQUOT_CRC_OFF);
-}
-
 void
 xfs_verify_recalc_crc(
 	struct xfs_buf *bp)
@@ -510,14 +483,10 @@ write_cur(void)
 		skip_crc = true;
 
 	if (!skip_crc) {
-		if (iocur_top->ino_buf) {
-			libxfs_dinode_calc_crc(mp, iocur_top->data);
-			iocur_top->ino_crc_ok = 1;
-		} else if (iocur_top->dquot_buf) {
-			xfs_update_cksum(iocur_top->data,
-					 sizeof(struct xfs_dqblk),
-					 XFS_DQUOT_CRC_OFF);
-		}
+		if (iocur_top->ino_buf)
+			xfs_inode_set_crc(iocur_top->bp);
+		else if (iocur_top->dquot_buf)
+			xfs_dquot_set_crc(iocur_top->bp);
 	}
 	if (iocur_top->bbmap)
 		write_cur_bbs();
@@ -545,7 +514,7 @@ set_cur(
 	struct xfs_buf	*bp;
 	xfs_ino_t	dirino;
 	xfs_ino_t	ino;
-	__uint16_t	mode;
+	uint16_t	mode;
 	const struct xfs_buf_ops *ops = type ? type->bops : NULL;
 
 	if (iocur_sp < 0) {
