@@ -1,19 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * Copyright (c) 2000-2002,2005 Silicon Graphics, Inc.
  * All Rights Reserved.
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it would be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write the Free Software Foundation,
- * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "libxfs.h"
@@ -35,10 +23,6 @@
 #include "slab.h"
 #include "rmap.h"
 
-#define	rounddown(x, y)	(((x)/(y))*(y))
-
-#define		XR_MAX_SECT_SIZE	(64 * 1024)
-
 /*
  * option tables for getsubopt calls
  */
@@ -764,7 +748,7 @@ main(int argc, char **argv)
 
 	chunks_pblock = mp->m_sb.sb_inopblock / XFS_INODES_PER_CHUNK;
 	max_symlink_blocks = libxfs_symlink_blocks(mp, XFS_SYMLINK_MAXLEN);
-	inodes_per_cluster = MAX(mp->m_sb.sb_inopblock,
+	inodes_per_cluster = max(mp->m_sb.sb_inopblock,
 			mp->m_inode_cluster_size >> mp->m_sb.sb_inodelog);
 
 	/*
@@ -862,9 +846,9 @@ main(int argc, char **argv)
 			rlim.rlim_cur = rlim.rlim_max;
 			setrlimit(RLIMIT_AS, &rlim);
 			/* use approximately 80% of rlimit to avoid overrun */
-			max_mem = MIN(max_mem, rlim.rlim_cur / 1280);
+			max_mem = min(max_mem, rlim.rlim_cur / 1280);
 		} else
-			max_mem = MIN(max_mem, (LONG_MAX >> 10) + 1);
+			max_mem = min(max_mem, (LONG_MAX >> 10) + 1);
 
 		if (verbose > 1)
 			do_log(
